package de.kif.backend.route

import de.kif.backend.authenticateOrRedirect
import de.kif.backend.isAuthenticated
import de.kif.backend.repository.PostRepository
import de.kif.backend.util.markdownToHtml
import de.kif.backend.view.MainTemplate
import de.kif.backend.view.MenuTemplate
import de.kif.common.model.Permission
import de.kif.common.model.Post
import io.ktor.application.call
import io.ktor.html.respondHtmlTemplate
import io.ktor.http.HttpStatusCode
import io.ktor.request.receiveParameters
import io.ktor.response.respond
import io.ktor.response.respondRedirect
import io.ktor.routing.Route
import io.ktor.routing.get
import io.ktor.routing.post
import io.ktor.util.toMap
import kotlinx.html.*

fun DIV.createPost(post: Post, editable: Boolean = false, additionalClasses: String = "") {
    var classes = "post"
    if (additionalClasses.isNotBlank()) {
        classes += " $additionalClasses"
    }
    div(classes) {
        attributes["data-id"] = post.id.toString()
        attributes["data-pinned"] = post.pinned.toString()

        a("/p/${post.url}", classes = "post-name") {
            +post.name
        }
        if (editable) {
            a("/post/${post.id}", classes = "post-edit") {
                i("material-icons") { +"edit" }
            }
        }
        div("post-content") {
            unsafe {
                raw(markdownToHtml(post.content))
            }
        }
    }
}

fun Route.overview() {
    get("") {
        val user = isAuthenticated(Permission.POST)
        val editable = user != null

        val postList = PostRepository.all().asReversed()

        call.respondHtmlTemplate(MainTemplate()) {
            menuTemplate {
                this.user = user
                active = MenuTemplate.Tab.BOARD
            }
            content {
                div("overview") {
                    div("overview-main") {
                        for (post in postList) {
                            createPost(post, editable, "overview-post")
                        }
                    }
                    div("overview-side") {
                        if (editable) {
                            div("overview-new") {
                                a("post/new", classes = "form-btn") {
                                    +"New"
                                }
                            }
                        }
                        div("overview-twitter") {
                            +"The Twitter Wall"
                        }
                    }
                }
            }
        }
    }

    get("/p/{url}") {
        val user = isAuthenticated(Permission.POST)
        val editable = user != null

        val post = PostRepository.getByUrl(call.parameters["url"] ?: "")

        if (post == null) {
            call.respond(HttpStatusCode.NotFound, "Not found")
            return@get
        }

        call.respondHtmlTemplate(MainTemplate()) {
            menuTemplate {
                this.user = user
                active = MenuTemplate.Tab.BOARD
            }
            content {
                div("overview") {
                    createPost(post, editable)
                }
            }
        }
    }

    get("/post/{id}") {
        authenticateOrRedirect(Permission.POST) { user ->
            val postId = call.parameters["id"]?.toLongOrNull() ?: return@get
            val editPost = PostRepository.get(postId) ?: return@get
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.BOARD
                }
                content {
                    h1 { +"Edit post" }
                    div("post-edit-container") {
                        div("post-edit-left") {
                            form(method = FormMethod.post) {
                                div("form-group") {
                                    label {
                                        htmlFor = "name"
                                        +"Name"
                                    }
                                    input(
                                        name = "name",
                                        classes = "form-control"
                                    ) {
                                        id = "name"
                                        placeholder = "Name"
                                        value = editPost.name
                                    }
                                }

                                div("form-group") {
                                    label {
                                        htmlFor = "url"
                                        +"Url"
                                    }
                                    input(
                                        name = "url",
                                        classes = "form-control"
                                    ) {
                                        id = "url"
                                        placeholder = "Url"
                                        value = editPost.url
                                    }
                                }

                                div("form-switch-group") {
                                    div("form-group form-switch") {
                                        input(
                                            name = "pinned",
                                            classes = "form-control",
                                            type = InputType.checkBox
                                        ) {
                                            id = "pinned"
                                            checked = editPost.pinned
                                        }
                                        label {
                                            htmlFor = "pinned"
                                            +"Pinned"
                                        }
                                    }
                                }

                                div("form-group") {
                                    label {
                                        htmlFor = "content"
                                        +"Content"
                                    }
                                    textArea(rows = "10", classes = "form-control") {
                                        name = "content"
                                        id = "content"

                                        +editPost.content
                                    }
                                }

                                div("form-group") {
                                    a("/") {
                                        button(classes = "form-btn") {
                                            +"Cancel"
                                        }
                                    }
                                    button(type = ButtonType.submit, classes = "form-btn btn-primary") {
                                        +"Save"
                                    }
                                }
                            }
                            a("/post/${editPost.id}/delete") {
                                button(classes = "form-btn btn-danger") {
                                    +"Delete"
                                }
                            }
                        }
                        div("post-edit-right post-content") {

                        }
                    }
                }
            }
        }
    }

    post("/post/{id}") {
        authenticateOrRedirect(Permission.POST) { user ->
            val postId = call.parameters["id"]?.toLongOrNull() ?: return@post
            val params = call.receiveParameters().toMap().mapValues { (_, list) ->
                list.firstOrNull()
            }
            var post = PostRepository.get(postId) ?: return@post

            params["name"]?.let { post = post.copy(name = it) }
            params["url"]?.let { post = post.copy(url = it) }
            params["content"]?.let { post = post.copy(content = it) }
            params["pinned"]?.let { post = post.copy(pinned = it == "on") }

            PostRepository.update(post)

            call.respondRedirect("/")
        }
    }

    get("/post/new") {
        authenticateOrRedirect(Permission.POST) { user ->
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.BOARD
                }
                content {
                    h1 { +"Create post" }
                    div("post-edit-container") {
                        div("post-edit-left") {
                            form(method = FormMethod.post) {
                                div("form-group") {
                                    label {
                                        htmlFor = "name"
                                        +"Name"
                                    }
                                    input(
                                        name = "name",
                                        classes = "form-control"
                                    ) {
                                        id = "name"
                                        placeholder = "Name"
                                        value = ""
                                    }
                                }
                                div("form-group") {
                                    label {
                                        htmlFor = "url"
                                        +"Url"
                                    }
                                    input(
                                        name = "url",
                                        classes = "form-control"
                                    ) {
                                        id = "url"
                                        placeholder = "Url"
                                        value = Post.generateUrl()
                                    }
                                }

                                div("form-switch-group") {
                                    div("form-group form-switch") {
                                        input(
                                            name = "pinned",
                                            classes = "form-control",
                                            type = InputType.checkBox
                                        ) {
                                            id = "pinned"
                                            checked = false
                                        }
                                        label {
                                            htmlFor = "pinned"
                                            +"Pinned"
                                        }
                                    }
                                }

                                div("form-group") {
                                    label {
                                        htmlFor = "content"
                                        +"Content"
                                    }
                                    textArea(rows = "10", classes = "form-control") {
                                        name = "content"
                                        id = "content"

                                        +""
                                    }
                                }

                                div("form-group") {
                                    a("/") {
                                        button(classes = "form-btn") {
                                            +"Cancel"
                                        }
                                    }
                                    button(type = ButtonType.submit, classes = "form-btn btn-primary") {
                                        +"Create"
                                    }
                                }
                            }
                        }
                        div("post-edit-right post-content") {

                        }
                    }
                }
            }
        }
    }

    post("/post/new") {
        authenticateOrRedirect(Permission.POST) { user ->
            val params = call.receiveParameters().toMap().mapValues { (_, list) ->
                list.firstOrNull()
            }

            val name = params["name"] ?: return@post
            val content = params["content"] ?: return@post
            val url = params["url"] ?: return@post
            val pinned = params["pinned"] == "on"

            val post = Post(null, name, content, url, pinned)

            PostRepository.create(post)

            call.respondRedirect("/")
        }
    }

    get("/post/{id}/delete") {
        authenticateOrRedirect(Permission.POST) { user ->
            val postId = call.parameters["id"]?.toLongOrNull() ?: return@get

            PostRepository.delete(postId)

            call.respondRedirect("/")
        }
    }
}
